@@ -36,17 +36,13 @@ module Sequel
       end
 
       module ClassMethods
-        # Setup the datastructure used to hold the prepared statements in the subclass.
-        def inherited(subclass)
-          super
-          subclass.instance_variable_set(:@prepared_statements, :insert=>{}, :insert_select=>{}, :update=>{}, :lookup_sql=>{}, :fixed=>{})
-        end
+        Plugins.inherited_instance_variables(self, :@prepared_statements=>lambda{|v| {:insert=>{}, :insert_select=>{}, :update=>{}, :lookup_sql=>{}, :fixed=>{}}})
 
         private
 
         # Create a prepared statement based on the given dataset with a unique name for the given
         # type of query and values.
-        def prepare_statement(ds, type, vals={})
+        def prepare_statement(ds, type, vals=OPTS)
           ps = ds.prepare(type, :"smpsp_#{NEXT.call}", vals)
           ps.log_sql = true
           ps
